
module top(
    
    input sw1,//L25
    input sw2,//R21
    input sw3,//R22
    input sw4,//T22
    output led_d1,
    output led_d2,
    output led_d3,
    output led_d4,
    output sfp_tx_disable, 
    output [15:0]   debug_vector_o,
    input sfp_los //r23
    
);
    
    assign sfp_tx_disable = 1'b0;

    wire        xgmii_tx_pll_lock_o;
    wire        s_lock_o;
    wire [7:0]  xgmii_txc;
    wire [63:0] xgmii_txd;
    wire [7:0]  xgmii_rxc;
    wire [63:0] xgmii_rxd;
    wire        block_lock; 
    wire        hi_ber; 
    wire        pcs_status;
    wire        clear_ber_count;
    wire [5:0]  ber_count; 
    wire        clear_errored_block_count;
    wire [7:0]  errored_block_count; 
    wire        ref_clk;
    wire        clk_out0_o;
    wire        clk_out1_o;
    wire        serdes_pll_lock;
    
    
    wire          rx_mac_clk;
    wire          rx_mac_valid;
    wire [63:0]   rx_mac_data;
    wire [7:0]    rx_mac_byte;
    wire          rx_mac_last;
    wire          rx_mac_error;
    wire          rx_statistics_valid;
    wire [19:0]   rx_statistics_vector;

    wire          tx_mac_clk;
    reg           tx_mac_valid;
    reg  [63:0]   tx_mac_data;
    wire [7:0]    tx_mac_byte;
    wire          tx_mac_last;
    wire          tx_mac_error;
    wire          tx_mac_ready;
    wire          tx_statistics_valid;
    wire [18:0]   tx_statistics_vector;

    wire          local_fault;
    wire          remote_fault;

    reg [1:0] c_tx_state;
    reg [1:0] n_tx_state;
    localparam S_TX_IDLE = 2'b00;
    localparam S_TX_DATA = 2'b01;
    localparam S_TX_LAST = 2'b10;

    reg tx_trigger;
    reg [3:0] idle_cnt;
    localparam FRM_LGT = 11'd999;
    localparam INIT_TX_DATA = 64'hafaeadacabaaa9a8;
    reg [10:0] byte_cnt;

    SerDes_Top u_SerDes_Top(
        .Ten_Giga_Serial_Ethernet_Top_xgmii_rxc_o(xgmii_rxc), //output [7:0] Ten_Giga_Serial_Ethernet_Top_xgmii_rxc_o
        .Ten_Giga_Serial_Ethernet_Top_xgmii_rxd_o(xgmii_rxd), //output [63:0] Ten_Giga_Serial_Ethernet_Top_xgmii_rxd_o
        .Ten_Giga_Serial_Ethernet_Top_ref_clk_o(ref_clk), //output Ten_Giga_Serial_Ethernet_Top_ref_clk_o
        .Ten_Giga_Serial_Ethernet_Top_clk_out0_o(clk_out0_o), //output Ten_Giga_Serial_Ethernet_Top_clk_out0_o
        .Ten_Giga_Serial_Ethernet_Top_clk_out1_o(clk_out1_o), //output Ten_Giga_Serial_Ethernet_Top_clk_out1_o
        .Ten_Giga_Serial_Ethernet_Top_block_lock_o(block_lock), //output Ten_Giga_Serial_Ethernet_Top_block_lock_o
        .Ten_Giga_Serial_Ethernet_Top_hi_ber_o(hi_ber), //output Ten_Giga_Serial_Ethernet_Top_hi_ber_o
        .Ten_Giga_Serial_Ethernet_Top_pcs_status_o(pcs_status), //output Ten_Giga_Serial_Ethernet_Top_pcs_status_o
        .Ten_Giga_Serial_Ethernet_Top_ber_count_o(ber_count), //output [5:0] Ten_Giga_Serial_Ethernet_Top_ber_count_o
        .Ten_Giga_Serial_Ethernet_Top_errored_block_count_o(errored_block_count), //output [7:0] Ten_Giga_Serial_Ethernet_Top_errored_block_count_o
        .Ten_Giga_Serial_Ethernet_Top_debug_vector_o(debug_vector_o), //output [15:0] Ten_Giga_Serial_Ethernet_Top_debug_vector_o
        .Ten_Giga_Serial_Ethernet_Top_rx_rstn_i(sw3), //input Ten_Giga_Serial_Ethernet_Top_rx_rstn_i
        .Ten_Giga_Serial_Ethernet_Top_tx_rstn_i(sw4), //input Ten_Giga_Serial_Ethernet_Top_tx_rstn_i
        .Ten_Giga_Serial_Ethernet_Top_signal_detect_i(1'b1), //input Ten_Giga_Serial_Ethernet_Top_signal_detect_i
        .Ten_Giga_Serial_Ethernet_Top_xgmii_rx_clk_i(xgmii_rx_clk), //input Ten_Giga_Serial_Ethernet_Top_xgmii_rx_clk_i
        .Ten_Giga_Serial_Ethernet_Top_xgmii_tx_clk_i(xgmii_tx_clk), //input Ten_Giga_Serial_Ethernet_Top_xgmii_tx_clk_i
        .Ten_Giga_Serial_Ethernet_Top_xgmii_rx_clk_ready_i(1'b1), //input Ten_Giga_Serial_Ethernet_Top_xgmii_rx_clk_ready_i
        .Ten_Giga_Serial_Ethernet_Top_xgmii_tx_clk_ready_i(1'b1), //input Ten_Giga_Serial_Ethernet_Top_xgmii_tx_clk_ready_i
        .Ten_Giga_Serial_Ethernet_Top_xgmii_txc_i(xgmii_txc), //input [7:0] Ten_Giga_Serial_Ethernet_Top_xgmii_txc_i
        .Ten_Giga_Serial_Ethernet_Top_xgmii_txd_i(xgmii_txd), //input [63:0] Ten_Giga_Serial_Ethernet_Top_xgmii_txd_i
        .Ten_Giga_Serial_Ethernet_Top_clear_ber_count_i(clear_ber_count), //input Ten_Giga_Serial_Ethernet_Top_clear_ber_count_i
        .Ten_Giga_Serial_Ethernet_Top_clear_errored_block_count_i(clear_errored_block_count) //input Ten_Giga_Serial_Ethernet_Top_clear_errored_block_count_i
    );


    assign clear_ber_count = 1'b0;
    assign clear_errored_block_count = 1'b0;
    assign xgmii_rx_clk = ref_clk;
    assign xgmii_tx_clk = ref_clk;
    assign serdes_pll_lock = debug_vector_o[0];

  //-------Instantiate 10G Ethernet MAC IP----------
    Ten_Giga_Ethernet_MAC_Top u_Ten_Giga_Ethernet_MAC_Top (
        //rstn
        .rx_rstn_i(block_lock),
        .tx_rstn_i(block_lock),
        //XGMII
        .xgmii_rx_clk_i(xgmii_rx_clk),
        .xgtx_clk_i(xgmii_tx_clk),
        
        .xgmii_rxc_i(xgmii_rxc),
        .xgmii_rxd_i(xgmii_rxd),
        .xgmii_txc_o(xgmii_txc),
        .xgmii_txd_o(xgmii_txd),
     
        //Config IF
        .rx_fcs_fwd_ena_i(1'b0),
        .rx_jumbo_ena_i(1'b1),
        .tx_fcs_fwd_ena_i(1'b0),
        .tx_fault_ena_i(1'b0),
        .tx_ifg_delay_ena_i(1'b0),
        .tx_ifg_delay_i(8'd3),
        //User IF
        .rx_mac_clk_o(rx_mac_clk),
        .rx_mac_valid_o(rx_mac_valid),
        .rx_mac_data_o(rx_mac_data),
        .rx_mac_byte_o(rx_mac_byte),
        .rx_mac_last_o(rx_mac_last),
        .rx_mac_error_o(rx_mac_error),
        .rx_statistics_valid_o(rx_statistics_valid),
        .rx_statistics_vector_o(rx_statistics_vector),
        
        .tx_mac_clk_o(tx_mac_clk),
        .tx_mac_valid_i(tx_mac_valid),
        .tx_mac_data_i(tx_mac_data),
        .tx_mac_byte_i(tx_mac_byte),
        .tx_mac_last_i(tx_mac_last),
        .tx_mac_error_i(tx_mac_error),
        .tx_mac_ready_o(tx_mac_ready),
        .tx_statistics_valid_o(tx_statistics_valid),
        .tx_statistics_vector_o(tx_statistics_vector),

        .local_fault_o(local_fault),
        .remote_fault_o(remote_fault)

    );
    




    //------tx user if--------

    always@(posedge tx_mac_clk or negedge block_lock) begin
        if (!block_lock) begin
            tx_trigger <= 1'b0;
            idle_cnt <= 4'd0;
            byte_cnt <= 11'd0;
            tx_mac_data <= 64'b0;
            tx_mac_valid <= 1'b0;
        end
        else begin
            tx_trigger <= &idle_cnt;
            if (c_tx_state == S_TX_IDLE) begin
                idle_cnt <= idle_cnt + 1;
            end
            else begin
                idle_cnt <= 4'd0;
            end
            if (c_tx_state == S_TX_IDLE) begin
                byte_cnt <= FRM_LGT;
            end
            else if ((c_tx_state == S_TX_DATA) && tx_mac_ready) begin
                byte_cnt <= byte_cnt - 8;
            end
            if (c_tx_state == S_TX_IDLE) begin
                tx_mac_data <= INIT_TX_DATA;
            end
            else if (tx_mac_ready) begin
                tx_mac_data[8*0+:8] <= tx_mac_data[8*0+:8] + 8'h8;
                tx_mac_data[8*1+:8] <= tx_mac_data[8*1+:8] + 8'h8;
                tx_mac_data[8*2+:8] <= tx_mac_data[8*2+:8] + 8'h8;
                tx_mac_data[8*3+:8] <= tx_mac_data[8*3+:8] + 8'h8;
                tx_mac_data[8*4+:8] <= tx_mac_data[8*4+:8] + 8'h8;
                tx_mac_data[8*5+:8] <= tx_mac_data[8*5+:8] + 8'h8;
                tx_mac_data[8*6+:8] <= tx_mac_data[8*6+:8] + 8'h8;
                tx_mac_data[8*7+:8] <= tx_mac_data[8*7+:8] + 8'h8;
            end
            if (c_tx_state == S_TX_IDLE && n_tx_state == S_TX_DATA) begin
                tx_mac_valid <= 1'b1;
            end
            else if (c_tx_state == S_TX_LAST) begin
                tx_mac_valid <= 1'b0;
            end
        end
    end     

    assign tx_mac_last = (c_tx_state == S_TX_LAST);
    assign tx_mac_error = 1'b0;
        

    always@(posedge tx_mac_clk or negedge block_lock) begin
        if (!block_lock) begin
            c_tx_state <= S_TX_IDLE;
        end
        else begin
            c_tx_state <= n_tx_state;
        end
    end

    always@(*) begin
        n_tx_state = c_tx_state;
        case(c_tx_state)
        S_TX_IDLE:
            if (tx_trigger) begin
                n_tx_state = S_TX_DATA;
            end
        S_TX_DATA:
            if (byte_cnt <= 16) begin
                n_tx_state = S_TX_LAST;
            end
        S_TX_LAST:
            n_tx_state = S_TX_IDLE;
        endcase
    end

    assign tx_mac_byte = (c_tx_state == S_TX_DATA) ? 8'b1111_1111 :                    
                         (c_tx_state == S_TX_LAST) ? ((byte_cnt == 8) ? 8'b1111_1111  :
                                                      (byte_cnt == 7) ? 8'b0111_1111  :
                                                      (byte_cnt == 6) ? 8'b0011_1111  :
                                                      (byte_cnt == 5) ? 8'b0001_1111  :
                                                      (byte_cnt == 4) ? 8'b0000_1111  :
                                                      (byte_cnt == 3) ? 8'b0000_0111  :
                                                      (byte_cnt == 2) ? 8'b0000_0011  :
                                                      (byte_cnt == 1) ? 8'b0000_0001  : 
                                                                        8'b1111_1111) :
                                                       8'b1111_1111;
            
        
endmodule

