`timescale 1ns/1ps

module tb;

//------Simulate Serdes Clock and XGMII Clock--------------
localparam hs_period = 0.097; 
reg hs_clk = 0; 

initial begin
    forever #(hs_period) hs_clk = ~hs_clk;
end

reg serdes_clk = 0;
reg ref_clk = 0;

reg [6:0] serdes_clk_cnt = 0;
reg [6:0] ref_clk_cnt = 0;

always@(hs_clk) begin
    if (serdes_clk_cnt == 31) begin
        serdes_clk_cnt <= 0;
        serdes_clk <= ~serdes_clk;
    end
    else begin
        serdes_clk_cnt <= serdes_clk_cnt + 1;
    end

    if (ref_clk_cnt == 32) begin
        ref_clk_cnt <= 0;
        ref_clk <= ~ref_clk;
    end
    else begin
        ref_clk_cnt <= ref_clk_cnt + 1;
    end
end


wire [7:0] xgmii_txc;
wire [63:0] xgmii_txd;
wire [7:0] xgmii_rxc;
wire [63:0] xgmii_rxd;
wire xgmii_clk;
wire ref_clk_o;
wire block_lock;
wire [79:0] serdes_txdata;
wire [87:0] serdes_rxdata;
reg serdes_pll_lock = 1'b0;
reg serdes_rx_lock = 1'b0;
reg serdes_data_valid = 1'b0;
wire          rx_mac_clk;
wire          rx_mac_valid;
wire [63:0]   rx_mac_data;
wire [7:0]    rx_mac_byte;
wire          rx_mac_last;
wire          rx_mac_error;
wire          rx_statistics_valid;
wire [19:0]   rx_statistics_vector;

wire          tx_mac_clk;
reg           tx_mac_valid;
reg  [63:0]   tx_mac_data;
wire [7:0]    tx_mac_byte;
wire          tx_mac_last;
wire          tx_mac_error;
wire          tx_mac_ready;
wire          tx_statistics_valid;
wire [18:0]   tx_statistics_vector;

wire          local_fault;
wire          remote_fault;

reg [1:0] c_tx_state;
reg [1:0] n_tx_state;
localparam S_TX_IDLE = 2'b00;
localparam S_TX_DATA = 2'b01;
localparam S_TX_LAST = 2'b10;

reg tx_trigger;
reg [3:0] idle_cnt;
localparam FRM_LGT = 11'd999;
localparam INIT_TX_DATA = 64'hafaeadacabaaa9a8;
reg [10:0] byte_cnt;

assign xgmii_clk = ref_clk_o;
assign serdes_rxdata = (serdes_data_valid) ? serdes_txdata : 88'b0;

//-------Instantiate 10G Serial Ethernet IP----------
Ten_Giga_Serial_Ethernet_Top Ten_Giga_Serial_Ethernet
(  
    .rx_rstn_i(1'b1),                    //input           
    .tx_rstn_i(1'b1),                    //input           
    .signal_detect_i(1'b1),              //input           
    .xgmii_rx_clk_i(xgmii_clk),          //input           
    .xgmii_tx_clk_i(xgmii_clk),          //input           
    .xgmii_rx_clk_ready_i(1'b1),         //input           
    .xgmii_tx_clk_ready_i(1'b1),         //input           
    .xgmii_txc_i(xgmii_txc),             //input [7:0]     
    .xgmii_txd_i(xgmii_txd),             //input [63:0]    
    .xgmii_rxc_o(xgmii_rxc),             //output [7:0]    
    .xgmii_rxd_o(xgmii_rxd),             //output [63:0]   
    .ref_clk_o(ref_clk_o),               //output          
    .clk_out0_o(),                       //output          
    .clk_out1_o(),                       //output          
    .block_lock_o(block_lock),           //output          
    .hi_ber_o(),                         //output          
    .pcs_status_o(),                     //output          
    .clear_ber_count_i(1'b0),            //input           
    .ber_count_o(),                      //output [5:0]    
    .clear_errored_block_count_i(1'b0),  //input           
    .errored_block_count_o(),            //output [7:0]    
    .debug_vector_o(),                   //output [15:0]   
    .serdes_pma_rx_lock_i(serdes_rx_lock),         //input           
    .serdes_qpll0_pll_lock_i(serdes_pll_lock),      //input           
    .serdes_qpll1_pll_lock_i(serdes_pll_lock),      //input           
    .serdes_cpll_pll_lock_i(serdes_pll_lock),       //input           
    .serdes_pcs_tx_clk_i(serdes_clk),    //input           
    .serdes_pcs_tx_clk_o(),              //output          
    .serdes_txdata_o(serdes_txdata),     //output [79:0]   
    .serdes_pcs_rx_clk_i(serdes_clk),    //input           
    .serdes_pcs_rx_clk_o(),              //output          
    .serdes_rxdata_i(serdes_rxdata),     //input [87:0]    
    .serdes_refclk0_i(ref_clk),          //input           
    .serdes_refclk1_i(ref_clk),          //input           
    .serdes_pma_rstn_o(),                //output          
    .serdes_pcs_tx_rst_o(),              //output          
    .serdes_pcs_rx_rst_o(),              //output          
    .serdes_pcs_rx_rden_o(),             //output          
    .serdes_pcs_rx_aempty_i(),           //input           
    .serdes_pcs_tx_wren_o(),             //output          
    .serdes_pcs_tx_afull_i(),            //input           
    .serdes_astat_i()                    //input [5:0]     
);


//-------Instantiate 10G Ethernet MAC IP----------
Ten_Giga_Ethernet_MAC_Top Ten_Giga_Ethernet_MAC_Top   (
    //rstn
    .rx_rstn_i(block_lock),
    .tx_rstn_i(block_lock),
    //XGMII
    .xgmii_rx_clk_i(xgmii_clk),
    .xgtx_clk_i(xgmii_clk),
    
    .xgmii_rxc_i(xgmii_rxc),
    .xgmii_rxd_i(xgmii_rxd),
    .xgmii_txc_o(xgmii_txc),
    .xgmii_txd_o(xgmii_txd),
 
    //Config IF
    .rx_fcs_fwd_ena_i(1'b0),
    .rx_jumbo_ena_i(1'b1),
    .tx_fcs_fwd_ena_i(1'b0),
    .tx_fault_ena_i(1'b0),
    .tx_ifg_delay_ena_i(1'b0),
    .tx_ifg_delay_i(8'd3),
    //User IF
    .rx_mac_clk_o(rx_mac_clk),
    .rx_mac_valid_o(rx_mac_valid),
    .rx_mac_data_o(rx_mac_data),
    .rx_mac_byte_o(rx_mac_byte),
    .rx_mac_last_o(rx_mac_last),
    .rx_mac_error_o(rx_mac_error),
    .rx_statistics_valid_o(rx_statistics_valid),
    .rx_statistics_vector_o(rx_statistics_vector),
    
    .tx_mac_clk_o(tx_mac_clk),
    .tx_mac_valid_i(tx_mac_valid),
    .tx_mac_data_i(tx_mac_data),
    .tx_mac_byte_i(tx_mac_byte),
    .tx_mac_last_i(tx_mac_last),
    .tx_mac_error_i(tx_mac_error),
    .tx_mac_ready_o(tx_mac_ready),
    .tx_statistics_valid_o(tx_statistics_valid),
    .tx_statistics_vector_o(tx_statistics_vector),

    .local_fault_o(local_fault),
    .remote_fault_o(remote_fault)

);

//------tx user if--------

always@(posedge tx_mac_clk or negedge block_lock) begin
    if (!block_lock) begin
        tx_trigger <= 1'b0;
        idle_cnt <= 4'd0;
        byte_cnt <= 11'd0;
        tx_mac_data <= 64'b0;
        tx_mac_valid <= 1'b0;
    end
    else begin
        tx_trigger <= &idle_cnt;
        if (c_tx_state == S_TX_IDLE) begin
            idle_cnt <= idle_cnt + 1;
        end
        else begin
            idle_cnt <= 4'd0;
        end
        if (c_tx_state == S_TX_IDLE) begin
            byte_cnt <= FRM_LGT;
        end
        else if ((c_tx_state == S_TX_DATA) && tx_mac_ready) begin
            byte_cnt <= byte_cnt - 8;
        end
        if (c_tx_state == S_TX_IDLE) begin
            tx_mac_data <= INIT_TX_DATA;
        end
        else if (tx_mac_ready) begin
            tx_mac_data[8*0+:8] <= tx_mac_data[8*0+:8] + 8'h8;
            tx_mac_data[8*1+:8] <= tx_mac_data[8*1+:8] + 8'h8;
            tx_mac_data[8*2+:8] <= tx_mac_data[8*2+:8] + 8'h8;
            tx_mac_data[8*3+:8] <= tx_mac_data[8*3+:8] + 8'h8;
            tx_mac_data[8*4+:8] <= tx_mac_data[8*4+:8] + 8'h8;
            tx_mac_data[8*5+:8] <= tx_mac_data[8*5+:8] + 8'h8;
            tx_mac_data[8*6+:8] <= tx_mac_data[8*6+:8] + 8'h8;
            tx_mac_data[8*7+:8] <= tx_mac_data[8*7+:8] + 8'h8;
        end
        if (c_tx_state == S_TX_IDLE && n_tx_state == S_TX_DATA) begin
            tx_mac_valid <= 1'b1;
        end
        else if (c_tx_state == S_TX_LAST) begin
            tx_mac_valid <= 1'b0;
        end
    end
end     

assign tx_mac_last = (c_tx_state == S_TX_LAST);
assign tx_mac_error = 1'b0;
    

always@(posedge tx_mac_clk or negedge block_lock) begin
    if (!block_lock) begin
        c_tx_state <= S_TX_IDLE;
    end
    else begin
        c_tx_state <= n_tx_state;
    end
end

always@(*) begin
    n_tx_state = c_tx_state;
    case(c_tx_state)
    S_TX_IDLE:
        if (tx_trigger) begin
            n_tx_state = S_TX_DATA;
        end
    S_TX_DATA:
        if (byte_cnt <= 16) begin
            n_tx_state = S_TX_LAST;
        end
    S_TX_LAST:
        n_tx_state = S_TX_IDLE;
    endcase
end

assign tx_mac_byte = (c_tx_state == S_TX_DATA) ? 8'b1111_1111 :                    
                     (c_tx_state == S_TX_LAST) ? ((byte_cnt == 8) ? 8'b1111_1111  :
                                                  (byte_cnt == 7) ? 8'b0111_1111  :
                                                  (byte_cnt == 6) ? 8'b0011_1111  :
                                                  (byte_cnt == 5) ? 8'b0001_1111  :
                                                  (byte_cnt == 4) ? 8'b0000_1111  :
                                                  (byte_cnt == 3) ? 8'b0000_0111  :
                                                  (byte_cnt == 2) ? 8'b0000_0011  :
                                                  (byte_cnt == 1) ? 8'b0000_0001  : 
                                                                    8'b1111_1111) :
                                                   8'b1111_1111;
        
    

initial begin
    #100;
    serdes_pll_lock = 1'b1;
    #100;
    serdes_rx_lock = 1'b1;
    #100;
    serdes_data_valid = 1'b1;
end

`ifdef VCS
initial begin
    $fsdbDumpfile("tb.fsdb");
    $fsdbDumpvars;
end
`endif


always@(posedge tx_mac_valid)
    $display("--------Send Frame--------");
    
always@(posedge rx_mac_valid)
    $display("--------Receive Frame--------");

initial begin
    wait(block_lock);
    $display("--------10G Serial Ethernet RX Block Lock--------");

    wait(~tx_mac_valid);
    repeat(10) begin
        wait(tx_mac_valid);
        wait(~tx_mac_valid);
    end
    force tx_mac_valid = 1'b0;
    #1000;
    
    `ifdef MODELSIM
        $stop;
    `endif
    `ifdef VCS
        $finish;
    `endif
end
    
    
    

endmodule







